@@ -1,10 +1,14 @@
 # coding:utf-8
 
 from __future__ import absolute_import, division, print_function, unicode_literals
+from . import test_utils
 from .. import source, lexer, diagnostic, ast, coverage
 from ..coverage import parser
 import unittest, sys, re, ast as pyast
 
+BytesOnly = test_utils.BytesOnly
+UnicodeOnly = test_utils.UnicodeOnly
+
 if sys.version_info >= (3,):
     def unicode(x): return x
 
@@ -132,14 +136,17 @@ class ParserTestCase(unittest.TestCase):
             if not only_if(version):
                 continue
 
-            ast = self.parser_for(code + "\n", version).file_input()
+            ast = self.parser_for(code, version).file_input()
             flat_ast = self.flatten_ast(ast)
             self.assertEqual({"ty": "Module", "body": expected_flat_ast},
                              flat_ast)
             self.match_loc(ast, loc_matcher, ast_slicer)
 
-            if version == sys.version_info[0:2] and validate_if():
-                python_ast = pyast.parse(code.replace("·", "\n") + "\n")
+            compatible_pyast_version = \
+                (sys.version_info[0:2] == (2, 7) or
+                 sys.version_info[0:2] == (3, 4))
+            if compatible_pyast_version and version == sys.version_info[0:2] and validate_if():
+                python_ast = pyast.parse(code.replace("·", "\n"))
                 flat_python_ast = self.flatten_python_ast(python_ast)
                 self.assertEqual({"ty": "Module", "body": expected_flat_ast},
                                  flat_python_ast)
@@ -201,6 +208,7 @@ class ParserTestCase(unittest.TestCase):
     ast_expr_1 = {"ty": "Expr", "value": {"ty": "Num", "n": 1}}
     ast_expr_2 = {"ty": "Expr", "value": {"ty": "Num", "n": 2}}
     ast_expr_3 = {"ty": "Expr", "value": {"ty": "Num", "n": 3}}
+    ast_expr_4 = {"ty": "Expr", "value": {"ty": "Num", "n": 4}}
 
     ast_x = {"ty": "Name", "id": "x", "ctx": None}
     ast_y = {"ty": "Name", "id": "y", "ctx": None}
@@ -240,14 +248,40 @@ class ParserTestCase(unittest.TestCase):
             "'foo'",
             "~~~~~ loc"
             "^ begin_loc"
-            "    ^ end_loc")
+            "    ^ end_loc",
+            only_if=lambda ver: ver >= (3,))
+        self.assertParsesExpr(
+            {"ty": "Str", "s": BytesOnly("foo")}, "'foo'",
+            only_if=lambda ver: ver < (3,))
+
+        self.assertParsesExpr(
+            {"ty": "Str", "s": BytesOnly(b"foo")},
+            "b'foo'",
+            "~~~~~~ loc"
+            "~~ begin_loc"
+            "     ^ end_loc",
+            # Python 3.4 for some reason produces a Bytes node where all other
+            # known versions produce Str.
+            validate_if=lambda: sys.version_info[:2] != (3, 4))
+
+        self.assertParsesExpr(
+            {"ty": "Str", "s": BytesOnly(b"foo")},
+            "'foo'",
+            "~~~~~ loc"
+            "^ begin_loc"
+            "    ^ end_loc",
+            only_if=lambda ver: ver < (3,))
 
         self.assertParsesExpr(
             {"ty": "Str", "s": "foobar"},
             "'foo' 'bar'",
             "~~~~~~~~~~~ loc"
             "^ begin_loc"
-            "          ^ end_loc")
+            "          ^ end_loc",
+            only_if=lambda ver: ver >= (3,))
+        self.assertParsesExpr(
+            {"ty": "Str", "s": BytesOnly("foobar")}, "'foo' 'bar'",
+            only_if=lambda ver: ver < (3,))
 
     def test_ident(self):
         self.assertParsesExpr(
@@ -831,6 +865,16 @@ class ParserTestCase(unittest.TestCase):
             "  ^^ dstar_loc"
             "~~~~~~ loc")
 
+        self.assertParsesExpr(
+            {"ty": "Call", "func": self.ast_x, "starargs": None, "kwargs": self.ast_z,
+             "args": [self.ast_1], "keywords": []},
+            "x(1, **z)")
+
+        self.assertParsesExpr(
+            {"ty": "Call", "func": self.ast_x, "starargs": None, "kwargs": None,
+             "args": [self.ast_1, self.ast_2], "keywords": []},
+            "x(1, 2,)")
+
         self.assertParsesExpr(
             {"ty": "Call", "func": self.ast_x, "starargs": None, "kwargs": None,
              "keywords": [], "args": [
@@ -1066,6 +1110,13 @@ class ParserTestCase(unittest.TestCase):
             "~~~~~~~~~~~~ 0.loc")
 
     def test_print(self):
+        self.assertParsesSuite(
+            [{"ty": "Print", "dest": None, "values": [], "nl": True}],
+            "print",
+            "~~~~~ 0.keyword_loc"
+            "~~~~~ 0.loc",
+            only_if=lambda ver: ver < (3, 0))
+
         self.assertParsesSuite(
             [{"ty": "Print", "dest": None, "values": [self.ast_1], "nl": True}],
             "print 1",
@@ -1410,6 +1461,16 @@ class ParserTestCase(unittest.TestCase):
             "          ~~~~~~~~~~~~~~~~~~~~~ 0.orelse.0.loc"
             "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 0.loc")
 
+        # Ensure elif nodes are in the correct order.
+        # See: https://github.com/m-labs/pythonparser/issues/7
+        self.assertParsesSuite(
+            [{"ty": "If", "test": self.ast_x, "body": [self.ast_expr_1], "orelse": [
+               {"ty": "If", "test": self.ast_y, "body": [self.ast_expr_2], "orelse": [
+                 {"ty": "If", "test": self.ast_z, "body": [self.ast_expr_3],
+                  "orelse": [self.ast_expr_4]}
+            ]}]}],
+            "if x:·  1·elif y:·  2·elif z:·  3·else:·  4")
+
     def test_while(self):
         self.assertParsesSuite(
             [{"ty": "While", "test": self.ast_x, "body": [self.ast_expr_1], "orelse": []}],
@@ -1675,6 +1736,36 @@ class ParserTestCase(unittest.TestCase):
             "~~~~~~~~~~~~~~~~~~ 0.loc",
             validate_if=lambda: sys.version_info >= (3, 0))
 
+        self.assertParsesSuite(
+            [{"ty": "ClassDef", "name": "x", "bases": [],
+              "keywords": [], "starargs": None, "kwargs": None,
+              "body": [{"ty": "Pass"}], "decorator_list": [{
+                "ty": "Attribute", "value": self.ast_x, "attr": "y", "ctx": None}]}],
+            "@x.y·class x:·  pass",
+            validate_if=lambda: sys.version_info >= (3, 0))
+
+        self.assertParsesSuite(
+            [{"ty": "FunctionDef", "name": "x",
+              "args": {"ty": "arguments", "args": [], "defaults": [],
+                       "kwonlyargs": [], "kw_defaults": [],
+                       "kwarg": None, "vararg": None},
+              "returns": None,
+              "body": [{"ty": "Pass"}], "decorator_list": [{
+                "ty": "Attribute", "attr": "z", "ctx": None, "value": {
+                  "ty": "Attribute", "attr": "y", "ctx": None, "value":
+                    self.ast_x}}]}],
+            "@x.y.z·def x():·  pass",
+            "^ 0.at_locs.0"
+            " ~~~~~ 0.decorator_list.0.loc"
+            "    ^  0.decorator_list.0.dot_loc"
+            "     ^ 0.decorator_list.0.attr_loc"
+            " ~~~   0.decorator_list.0.value.loc"
+            "  ^    0.decorator_list.0.value.dot_loc"
+            "   ^   0.decorator_list.0.value.attr_loc"
+            " ^     0.decorator_list.0.value.value.loc"
+            "~~~~~~~~~~~~~~~~~~~~~~ 0.loc",
+            validate_if=lambda: sys.version_info >= (3, 0))
+
     #
     # FUNCTION AND LAMBDA ARGUMENTS
     #
@@ -1896,6 +1987,27 @@ class ParserTestCase(unittest.TestCase):
                  "starargs": None, "kwargs": None, "args": [self.ast_x], "keywords": []}}],
             "from __future__ import print_function·print(x)")
 
+    def test_future_unicode_literals(self):
+        self.assertParsesGen(
+            [{"ty": "ImportFrom",
+              "names": [{"ty": "alias", "name": "unicode_literals", "asname": None}],
+              "module": "__future__", "level": 0},
+             {"ty": "Expr", "value": {"ty": "Str", "s": UnicodeOnly("foo")}}],
+            "from __future__ import unicode_literals·'foo'",
+            only_if=lambda ver: ver < (3, 0))
+
+        # This test does not validate because unicode_literals import only
+        # affects subsequent string literals in pythonparser, whereas in ast it
+        # affects all string literals in the file.
+        self.assertParsesGen(
+            [{"ty": "Expr", "value": {"ty": "Str", "s": BytesOnly(b"foo")}},
+             {"ty": "ImportFrom",
+              "names": [{"ty": "alias", "name": "unicode_literals", "asname": None}],
+              "module": "__future__", "level": 0}],
+            "'foo'·from __future__ import unicode_literals",
+            only_if=lambda ver: ver == (2, 7), validate_if=lambda: False)
+
+
     #
     # DIAGNOSTICS
     #
